importPackage(java.lang);
importPackage(java.sql);
importPackage(java.util);
importPackage(java.io);
importPackage(Packages.javax.xml);
importPackage(Packages.javax.xml.parsers);
importPackage(Packages.javax.xml.transform);
importPackage(Packages.javax.xml.transform.dom);
importPackage(Packages.javax.xml.transform.stream);
importPackage(Packages.org.w3c.dom);
importPackage(Packages.com.invoqsystems.integration.remedy.scriptobjects);

var remedyURL = "http://localhost";
var remedyServerName = "REMEDYSD2k3";

function escapeMessage( msg )
{
  if ( msg == null )
  {
    return null;
  }
  msg = msg.replaceAll( "\n"," " );
  msg = msg.replaceAll( ".*java.lang.Exception: WS Fault: soapenv:Server.userException [|]+[ ]+MessageType: [0-9]+[ ]+MessageNum: [0-9]+[ ]+MessageText: ", "" );
  return msg;
}

function assignIncident( params )
{
  var user = params.get( "sdUser" );
  var pass = params.get( "sdPass" );
  var assignFields = params.get( "assignFields" );
  if ( user == null  || assignFields == null )
  {
    return "One of the required fields (sdUser, assignFields) has no value.";
  }
  else
  {
    var sdso = new ServiceDeskScriptObject();
    sdso.initClient( remedyURL, remedyServerName, user, pass, null, null, null );
    var msg = sdso.assignIncident( assignFields );
    return escapeMessage( msg );
  }
}

function addWorkInfo( params ) 
{
  var user = params.get( "sdUser" );
  var pass = params.get( "sdPass" );
  var addInfoFields = params.get( "addInfoFields" );
  if ( user == null  || addInfoFields == null )
  {
    return "One of the required fields (sdUser, addInfoFields) has no value.";
  }
  else
  {
    var sdso = new ServiceDeskScriptObject();
    sdso.initClient( remedyURL, remedyServerName, user, pass, null, null, null );
    return escapeMessage( sdso.addWorkInfo( addInfoFields ) );
  }
}

function createIncident( params ) 
{
  var user = params.get( "sdUser" );
  var pass = params.get( "sdPass" );
  var createFields = params.get( "createFields" );
  if ( user == null  || createFields == null )
  {
    return "One of the required fields (sdUser, createFields) has no value.";
  }
  else
  {
    var sdso = new ServiceDeskScriptObject();
    sdso.initClient( remedyURL, remedyServerName, user, pass, null, null, null );
    var msg = sdso.createIncident( createFields );
    return escapeMessage( msg );
  }
}

function ownIncident( params )
{
  var user = params.get( "sdUser" );
  var pass = params.get( "sdPass" );
  var ownFields = params.get( "ownFields" );
  if ( user == null  || ownFields == null )
  {
    return "One of the required fields (sdUser, ownFields) has no value.";
  }
  else
  {
    var sdso = new ServiceDeskScriptObject();
    sdso.initClient( remedyURL, remedyServerName, user, pass, null, null, null );
    var msg = sdso.ownIncident( ownFields );
    return escapeMessage( msg );
  }
}

/**
 * Query against Remedy WS for a count of all incidents that passes the passed in Qualification
 *
 * Parameters: a java.util.Map<String, String> containing 
 *             "Qualification", The qualification used in the Remedy search
  * Return: A java.lang.Integer 
 */
function getIncidentCount( params ) 
{
  var user = params.get( "sdUser" );
  var pass = params.get( "sdPass" );
  var qual = params.get( "Qualification" );
  if ( user == null  || qual == null )
  {
    return "One of the required fields (sdUser, Qualification) has no value.";
  }
  else
  {
    var sdso = new ServiceDeskScriptObject();
    sdso.initClient( remedyURL, remedyServerName, user, pass, null, null, null );
    return new Integer( sdso.queryIncidentIDs( qual, null, null ).size() );
  }
}

function getIncidentDetail( params )
{
  var user = params.get( "sdUser" );
  var pass = params.get( "sdPass" );
  var incidentNumber = params.get( "IncidentNumber" );  
  if ( user == null  || incidentNumber == null )
  {
    return "One of the required fields (sdUser, IncidentNumber) has no value.";
  }
  else
  {
    var sdso = new ServiceDeskScriptObject();
    sdso.initClient( remedyURL, remedyServerName, user, pass, null, null, null );  
    return sdso.queryIncidentDetail( incidentNumber );
  }
}

/**
 * Query against Remedy WS for header of all incidents that passes the passed in Qualification
 * A header consist of <Incident_Number, Status, Priority, Description>  
 *
 * Parameters: a java.util.Map<String, String> containing 
 *             "Qualification", The qualification used in the Remedy search
  * Return: Header information of the matched incidents.
 */
function getIncidentHeaders( params ) 
{
  var user = params.get( "sdUser" );
  var pass = params.get( "sdPass" );
  var qual = params.get( "Qualification" );
  if ( user == null  || qual == null )
  {
    return "One of the required fields (sdUser, Qualification) has no value.";
  }
  else
  {
    var sdso = new ServiceDeskScriptObject();
    try
    {
      sdso.initClient( remedyURL, remedyServerName, user, pass, null, null, null );
      return sdso.queryIncidentHeaders( qual, null, null );
    }
    catch (ex)
    {
      var msg = ex.toString();
      if ( msg.indexOf( "Qualification line error" ) != -1 )
      {
        return null;
      }
      return msg;
    }
  }
}

function queryCGM( params )
{
  var user = params.get( "sdUser" );
  var pass = params.get( "sdPass" );
  var maxLimit = params.get( "max_limit" );
  var qual = params.get( "Qualification" );
  var operation = params.get( "operation" );
  var service = params.get( "service" );

  if ( user == null  || qual == null || operation == null || service == null )
  {
    return "One of the required fields (sdUser, Qualification, Operation, Service) has no value.";
  }
  else
  {    
    var sdso = new ServiceDeskScriptObject();
    sdso.initClient( remedyURL, remedyServerName, user, pass, null, null, null );

    var valuesNode = "getListValues";
    try
    {
      return sdso.queryScriptObjectList( operation, service, valuesNode, qual, null, maxLimit );
    }
    catch (e)
    {
      return e.toString();
    }      
  }
}
function queryGenericList( params ) 
{
  var user = params.get( "sdUser" );
  var pass = params.get( "sdPass" );
  var serv = params.get( "service" );
  var maxLimit = params.get( "max_limit" );
  var qual = params.get( "Qualification" );
  if ( user == null  || qual == null || serv == null)
  {
    return "One of the required fields (sdUser, Qualification, service) has no value.";
  }
  else
  {
    var sdso = new ServiceDeskScriptObject();
    try
    {
      sdso.initClient( remedyURL, remedyServerName, user, pass, null, null, null );
      if ( "Customer".equalsIgnoreCase( serv ) )
      {
        return sdso.queryPeople( qual, null, maxLimit );
      }
      else if ( "Group".equalsIgnoreCase( serv ) )
      {
        return sdso.queryGroup( qual, null, maxLimit );
      }
      else if ( "VendorGroup".equalsIgnoreCase( serv ) )
      {
        qual = "('Vendor Group'=\"Yes\") AND (" + qual + ")";
        return sdso.queryGroup( qual, null, maxLimit );
      }
      else if ( "Assignee".equalsIgnoreCase( serv ) )
      {
        return sdso.queryGroupAssociation( qual, null, maxLimit );
      }
      else if ( "Reason".equals( serv ) )
      {
        var operation = "SYS_StatusReason_QueryList";
        var service = "APSDI_StatusReason_WS";
        var valuesNode = "getListValues";
        return sdso.queryScriptObjectList( operation, service, valuesNode, qual, null, maxLimit );
      }
      else if ( "Company".equals( serv ) )
      {
        qual = "('Company Type'=\"Operating Company\") AND (" + qual + ")";
        var operation = "APSDI_AllCom_QueryList";
        var service = "APSDI_COM_Company_WS";
        var valuesNode = "getListValues";
        return sdso.queryScriptObjectList( operation, service, valuesNode, qual, null, maxLimit );
      }
      else if ( "SatRating".equals( serv ) || "ResolutionMethod".equals( serv ) )
      {
        operation = "MG_MenuItems_QueryList";
        service = "APSDI_CFG_MenuItems_WS";
        valuesNode = "getListValues";
        return sdso.queryScriptObjectList( operation, service, valuesNode, qual, null, maxLimit );
      }
      else if ( "Cost".equals( serv ) )
      {
        operation = "FIN_Cost_QueryList";
        service = "APSDI_FIN_CostsAssociation_WS";
        valuesNode = "getListValues";
        return sdso.queryScriptObjectList( operation, service, valuesNode, qual, null, null );
      }
      else if ( "IDs".equals( serv ) )
      {
        operation = "HelpDesk_IDs_QueryList";
        service = "APSDI_HPD_HelpDesk_WS";
        valuesNode = "getListValues";
        return sdso.queryScriptObjectList( operation, service, valuesNode, qual, null, null );
      }
      else if ( "Relationships".equals( serv ) )
      {
        operation = "MG_Relation_QueryList";
        service = "APSDI_HPD_Associations";
        valuesNode = "getListValues";
        return sdso.queryScriptObjectList( operation, service, valuesNode, qual, null, null );
      }
    }
    catch (ex)
    {
      var msg = ex.toString();
      if ( msg.indexOf( "Qualification line error" ) != -1 )
      {
        return null;
      }
      return msg;
    }
  }
}

function queryNode( params )
{
  var user = params.get( "sdUser" );
  var pass = params.get( "sdPass" );
  var operation = params.get( "operation" );
  var webService = params.get( "webService" );
  var requestNS = params.get( "requestNS" );
  var responseNS = params.get( "responseNS" );
  var fields = params.get( "fields" );
  
  if ( user == null  ||
       operation == null || webService == null || 
       requestNS == null || responseNS == null || fields == null )
  {
    return "One of the required fields (sdUser, " +
    "operation, webService, requestNS, responseNS, fields) has no value.";
  }
  else
  {
    var sdso = new ServiceDeskScriptObject();
    sdso.initClient( remedyURL, remedyServerName, user, pass, null, null, null );
    return sdso.queryNode( operation, webService, requestNS, responseNS, fields );
  }
}

function queryTextValue( params )
{
  var user = params.get( "sdUser" );
  var pass = params.get( "sdPass" );
  var operation = params.get( "operation" );
  var webService = params.get( "webService" );
  var requestNS = params.get( "requestNS" );
  var responseNS = params.get( "responseNS" );
  var fields = params.get( "fields" );
  
  if ( user == null  ||
       operation == null || webService == null || 
       requestNS == null || responseNS == null || fields == null )
  {
    return "One of the required fields (sdUser, " +
    "operation, webService, requestNS, responseNS, fields) has no value.";
  }
  else
  {
    var sdso = new ServiceDeskScriptObject();
    sdso.initClient( remedyURL, remedyServerName, user, pass, null, null, null );
    return sdso.queryNode( operation, webService, requestNS, responseNS, fields );
  }
}

function queryWorkLogs( params )
{
  var user = params.get( "sdUser" );
  var pass = params.get( "sdPass" );
  var incidentNumber = params.get( "IncidentNumber" );
  if ( user == null  || incidentNumber == null )
  {
    return "One of the required fields (sdUser, IncidentNumber) has no value.";
  }
  else
  {
    var sdso = new ServiceDeskScriptObject();
    try
    {
      sdso.initClient( remedyURL, remedyServerName, user, pass, null, null, null );
      return sdso.queryWorkLogs( incidentNumber );
    }
    catch (ex)
    {
      ex.toString();
    }
  }
}

function resolveIncident( params ) 
{
  var user = params.get( "sdUser" );
  var pass = params.get( "sdPass" );
  var resolveFields = params.get( "resolveFields" );
  if ( user == null  || resolveFields == null )
  {
    return "One of the required fields (sdUser, resolveFields) has no value.";
  }
  else
  {
    var sdso = new ServiceDeskScriptObject();
    sdso.initClient( remedyURL, remedyServerName, user, pass, null, null, null );
    var response = "";
    try
    {
      response = sdso.resolveIncident( resolveFields );
    }
    catch (ex)
    {
      return ex.toString();
    }
    return response;
  }
}

function sendGenericRequest( params )
{
  var user = params.get( "sdUser" );
  var pass = params.get( "sdPass" );
  var operation = params.get( "operation" );
  var webService = params.get( "webService" );
  var requestNS = params.get( "requestNS" );
  var responseNS = params.get( "responseNS" );
  var fields = params.get( "fields" );
  
  if ( user == null  ||
       operation == null || webService == null || 
       requestNS == null || responseNS == null || fields == null )
  {
    return "One of the required fields (sdUser, " +
    "operation, webService, requestNS, responseNS, fields) has no value.";
  }
  else
  {
    var sdso = new ServiceDeskScriptObject();
    sdso.initClient( remedyURL, remedyServerName, user, pass, null, null, null );
    var msg = sdso.sendGenericRequest( operation, webService, requestNS, responseNS, fields );
    return escapeMessage( msg );
  }
}

// Sends request where XML contains more than one tier
//  complexMap is of type Map<String, Object> where object should be of type
//  String or Map<String, Object>
function sendComplexRequest( params )
{
  var user = params.get( "sdUser" );
  var pass = params.get( "sdPass" );
  var operation = params.get( "operation" );
  var webService = params.get( "webService" );
  var requestNS = params.get( "requestNS" );
  var responseNS = params.get( "responseNS" );
  var complexMap = params.get( "complexMap" );
  
  if ( user == null  ||
       operation == null || webService == null || 
       requestNS == null || responseNS == null || complexMap == null )
  {
    return "One of the required fields (sdUser, " +
    "operation, webService, requestNS, responseNS, complexMap) has no value.";
  }
  else
  {
    var sdso = new ServiceDeskScriptObject();
    sdso.initClient( remedyURL, remedyServerName, user, pass, null, null, null );
    var msg = sdso.sendComplexRequest( operation, webService, requestNS, responseNS, complexMap );
    return escapeMessage( msg );
  }
}

function updateIncident( params ) 
{
  var user = params.get( "sdUser" );
  var pass = params.get( "sdPass" );
  var incident = params.get( "incident" );
  if ( user == null  || incident == null )
  {
    return "One of the required fields (sdUser, Incident) has no value.";
  }
  else
  {
    var sdso = new ServiceDeskScriptObject();
    sdso.initClient( remedyURL, remedyServerName, user, pass, null, null, null );
    var operation = "HelpDesk_Modify_Service1";
    var webService = "APSDI_HPD_IncidentInterface_WS";
    var msg = sdso.sendGenericRequest( operation, webService, webService, webService, incident.getFields() );
    return escapeMessage( msg );
  }
}

/**
 * Used to validate a user SD credentials
 *
 * Returns: null if the validation was successful and a String with the error message otherwise
 */
function validateCreds(params)
{
  try
  {
    //ServiceDeskScriptObject( String url, String serverName, String user,
    //        String password, String authentication, String locale,
    //        String timeZone )
    var user = params.get( "sdUser" );
    var password = params.get( "sdPass" );    
    var sdso = new ServiceDeskScriptObject();
    sdso.initClient( remedyURL, remedyServerName, user, password, null, null, null );
    return sdso.validateCreds( user, password );
  }
  catch(e)
  {
    return e.toString();
  }
  return null; 
}
